@@ -14,12 +14,16 @@ import time
 import cv2
 import face_recognition
 import numpy as np
+from wotemu.monitor.utils import write_metric
 from wotemu.utils import consume_from_catalogue, wait_node
 from wotpy.wot.td import ThingDescription
 
 _QUEUE_MAXSIZE = 5
+_QUEUE_MAX_SECONDS = 10
 _DEFAULT_CAMERA_ID = "urn:org:fundacionctic:thing:wotemu:camera"
 _DEFAULT_FRAME_EVENT = "jpgVideoFrame"
+_PTZ_ACTION = "controlPTZ"
+_METRIC_LATENCY = "detection_latency"
 
 _DESCRIPTION = {
     "id": "urn:org:fundacionctic:thing:wotemu:detector",
@@ -45,7 +49,6 @@ _DESCRIPTION = {
             }
         },
         "latestDetections": {
-            "readOnly": True,
             "type": "array",
             "items": {
                 "type": "object",
@@ -79,7 +82,26 @@ def _detect(queue_item):
     return face_recognition.face_locations(frame_arr)
 
 
-async def _process_queue(detection_queue, store, timeout_get=3.0):
+async def _control_camera_ptz(wot, conf, camera_id, **kwargs):
+    try:
+        splitted = camera_id.split("::")
+        servient_host = splitted[0].strip()
+        thing_id = splitted[1].strip()
+
+        camera_thing = await consume_from_catalogue(
+            wot=wot,
+            port_catalogue=conf.port_catalogue,
+            servient_host=servient_host,
+            thing_id=thing_id)
+
+        await camera_thing.actions[_PTZ_ACTION].invoke(kwargs)
+    except Exception as ex:
+        _logger.warning("Failed updating PTZ controls: %s", repr(ex))
+
+
+async def _process_queue(wot, conf, detection_queue, prop, timeout_get=3.0):
+    store = list()
+
     while True:
         try:
             queue_item = await asyncio.wait_for(detection_queue.get(), timeout_get)
@@ -89,7 +111,12 @@ async def _process_queue(detection_queue, store, timeout_get=3.0):
 
         b64_jpg = queue_item["b64_jpg"]
         cam_id = queue_item["cam_id"]
-        unix_now = queue_item["unix_now"]
+        time_arrival = queue_item["time_arrival"]
+        time_capture = queue_item["time_capture"]
+
+        if (time.time() - time_arrival) >= _QUEUE_MAX_SECONDS:
+            _logger.info("Dropping old item (>%s s.)", _QUEUE_MAX_SECONDS)
+            continue
 
         _logger.debug(
             "[%s] Received B64-encoded JPG (%s) (%s KB)",
@@ -97,16 +124,30 @@ async def _process_queue(detection_queue, store, timeout_get=3.0):
             b64_jpg.__class__.__name__,
             round(sys.getsizeof(b64_jpg) / 1024.0, 1))
 
+        time_pre_detect = time.time()
+
         loop = asyncio.get_running_loop()
         detect = functools.partial(_detect, queue_item=queue_item)
         face_locations = await loop.run_in_executor(None, detect)
         no_faces = not face_locations or len(face_locations) == 0
 
+        time_detect = time.time()
+
         _logger.log(
             logging.DEBUG if no_faces else logging.INFO,
             "[%s] Face locations: %s",
             cam_id, face_locations)
 
+        await write_metric(key=_METRIC_LATENCY, data={
+            "time_capture": time_capture,
+            "time_arrival": time_arrival,
+            "time_detect": time_detect,
+            "latency_frame": time_arrival - time_capture,
+            "latency_detect": time_detect - time_arrival,
+            "detect_cost": time_detect - time_pre_detect,
+            "camera_id": cam_id
+        })
+
         if no_faces:
             continue
 
@@ -121,17 +162,28 @@ async def _process_queue(detection_queue, store, timeout_get=3.0):
             "cameraId": cam_id,
             "jpgB64": b64_jpg,
             "faceLocations": face_locations,
-            "unixTime": unix_now
+            "unixTime": time_capture
         })
 
+        prop.write(store)
+
+        asyncio.ensure_future(_control_camera_ptz(
+            wot=wot,
+            conf=conf,
+            camera_id=cam_id,
+            time_capture=time_capture,
+            time_arrival=time_arrival,
+            time_detect=time_detect))
+
 
 def _on_next(item, cam_id, detection_queue):
     try:
         _logger.debug("Putting new item in detection queue")
 
         detection_queue.put_nowait({
-            "b64_jpg": item.data,
-            "unix_now": int(time.time()),
+            "b64_jpg": item.data["b64_jpg"],
+            "time_capture": item.data["time_capture"],
+            "time_arrival": time.time(),
             "cam_id": cam_id
         })
     except asyncio.QueueFull:
@@ -223,8 +275,7 @@ async def _identity(val):
 
 async def app(wot, conf, loop, cameras):
     cameras = json.loads(cameras)
-    detection_queue = asyncio.Queue(maxsize=_QUEUE_MAXSIZE)
-    store = list()
+    detection_queue = asyncio.LifoQueue(maxsize=_QUEUE_MAXSIZE)
 
     _logger.info(
         "Producing Thing:\n%s",
@@ -236,10 +287,6 @@ async def app(wot, conf, loop, cameras):
         "cameras",
         functools.partial(_identity, val=cameras))
 
-    exposed_thing.set_property_read_handler(
-        "latestDetections",
-        functools.partial(_identity, val=store))
-
     exposed_thing.expose()
 
     _logger.debug(
@@ -248,8 +295,10 @@ async def app(wot, conf, loop, cameras):
 
     await asyncio.gather(
         _process_queue(
+            wot=wot,
+            conf=conf,
             detection_queue=detection_queue,
-            store=store),
+            prop=exposed_thing.properties["latestDetections"]),
         _subscribe_task(
             wot=wot,
             conf=conf,
